from __future__ import annotations
from yamloom.actions.utils import validate_choice

from dataclasses import dataclass
from typing import TYPE_CHECKING

from ...expressions import context, StringExpression
from ..._yamloom import Step
from ..._yamloom import action
from ..types import (
    Oboollike,
    Oboolstr,
    Ointlike,
    Ostr,
    Ostrlike,
    StringLike,
    BoolLike,
)

if TYPE_CHECKING:
    from collections.abc import Mapping

__all__ = ['setup_node', 'setup_pnpm', 'SetupNodeOutput', 'SetupPnpmOutput']


@dataclass(frozen=True)
class SetupNodeOutput:
    """Typed access to outputs produced by the setup_node step.

    Parameters
    ----------
    id
        The ``id`` of the setup_node step whose outputs should be referenced.
        This should match the ``id`` passed to :func:`setup_node`.

    Attributes
    ----------
    cache_hit
        A boolean value to indicate if a cache was hit.
    node_version
        The installed Node.js version.

    See Also
    --------
    GitHub repository: https://github.com/actions/setup-node
    """

    id: str

    @property
    def cache_hit(self) -> StringExpression:
        return context.steps[self.id].outputs['cache-hit']

    @property
    def node_version(self) -> StringExpression:
        return context.steps[self.id].outputs['node-version']


@dataclass(frozen=True)
class SetupPnpmOutput:
    """Typed access to outputs produced by the setup_pnpm step.

    Parameters
    ----------
    id
        The ``id`` of the setup_pnpm step whose outputs should be referenced.
        This should match the ``id`` passed to :func:`setup_pnpm`.

    Attributes
    ----------
    dest
        Expanded path of ``dest``.
    bin_dest
        Location of ``pnpm`` and ``pnpx`` command.

    See Also
    --------
    GitHub repository: https://github.com/pnpm/action-setup
    """

    id: str

    @property
    def dest(self) -> StringExpression:
        return context.steps[self.id].outputs.dest

    @property
    def bin_dest(self) -> StringExpression:
        return context.steps[self.id].outputs.bin_dest


def setup_node(
    *,
    name: Ostrlike = None,
    version: str = 'v6',
    node_version: Ostrlike = None,
    node_version_file: Ostrlike = None,
    check_latest: Oboollike = None,
    architecture: Ostrlike = None,
    token: Ostrlike = None,
    cache: Ostrlike = None,
    package_manager_cache: Oboollike = None,
    cache_dependency_path: Ostrlike = None,
    registry_url: Ostrlike = None,
    scope: Ostrlike = None,
    mirror: Ostrlike = None,
    mirror_token: Ostrlike = None,
    args: Ostrlike = None,
    entrypoint: Ostrlike = None,
    condition: Oboolstr = None,
    id: Ostr = None,  # noqa: A002
    env: Mapping[str, StringLike] | None = None,
    continue_on_error: Oboollike = None,
    timeout_minutes: Ointlike = None,
) -> Step:
    """Set up a Node.js environment.

    Parameters
    ----------
    name
        The name of the step to display on GitHub.
    version
        The branch, ref, or SHA of the action's repository to use.
    node_version
        Version spec of the version to use (e.g. ``12.x``).
    node_version_file
        File containing the version spec to use (e.g. ``.nvmrc``).
    architecture
        Target architecture for Node to use.
    check_latest
        Check for the latest available version that satisfies the version spec.
    registry_url
        Optional registry to set up for auth.
    scope
        Optional scope for authenticating against scoped registries.
    token
        Used to pull Node distributions from node-versions.
    cache
        Package manager for caching in the default directory. Supported values:
        ``npm``, ``yarn``, ``pnpm``.
    package_manager_cache
        Set to false to disable automatic caching.
    cache_dependency_path
        Path to a dependency file (package-lock.json, yarn.lock, etc.).
    mirror
        Alternative mirror to download Node.js binaries from.
    mirror_token
        Token used as Authorization header when fetching from the mirror.
    args
        The inputs for a Docker container which are passed to the container's entrypoint.
        This is a subkey of the ``with`` key of the generated step.
    entrypoint
        Overrides the Docker ENTRYPOINT in the action's Dockerfile or sets one if it was not
        specified. Accepts a single string defining the executable to run (note that this is
        different from Docker's ENTRYPOINT instruction which has both a shell and exec form).
        This is a subkey of the ``with`` key of the generated step.
    condition
        A boolean expression which must be met for the step to run. Note that this represents
        the ``if`` key in the actual YAML file.
    id
        A unique identifier for the step which can be referenced in expressions.
    env
        Used to specify environment variables for the step.
    continue_on_error
        Prevents the job from failing if this step fails.
    timeout_minutes
        The maximum number of minutes to let the step run before GitHub automatically
        cancels it (defaults to 360 if not specified).

    Returns
    -------
    Step
        The generated setup-node step.

    See Also
    --------
    GitHub repository: https://github.com/actions/setup-node
    """
    options: dict[str, object] = {
        'node-version': node_version,
        'node-version-file': node_version_file,
        'check-latest': check_latest,
        'architecture': architecture,
        'token': token,
        'cache': validate_choice('cache', cache, ['npm', 'yarn', 'pnpm']),
        'package-manager-cache': package_manager_cache,
        'cache-dependency-path': cache_dependency_path,
        'registry-url': registry_url,
        'scope': scope,
        'mirror': mirror,
        'mirror-token': mirror_token,
    }

    options = {key: value for key, value in options.items() if value is not None}

    if name is None:
        name = 'Setup Node'

    return action(
        name,
        'actions/setup-node',
        ref=version,
        with_opts=options or None,
        args=args,
        entrypoint=entrypoint,
        condition=condition,
        id=id,
        env=env,
        continue_on_error=continue_on_error,
        timeout_minutes=timeout_minutes,
    )


def setup_pnpm(
    *,
    name: Ostrlike = None,
    version: str = 'v4',
    pnpm_version: Ostrlike = None,
    dest: Ostrlike = None,
    run_install: StringLike | BoolLike | None = None,
    cache: Oboollike = None,
    cache_dependency_path: Ostrlike | list[StringLike] = None,
    package_json_file: Ostrlike = None,
    standalone: Oboollike = None,
    args: Ostrlike = None,
    entrypoint: Ostrlike = None,
    condition: Oboolstr = None,
    id: Ostr = None,  # noqa: A002
    env: Mapping[str, StringLike] | None = None,
    continue_on_error: Oboollike = None,
    timeout_minutes: Ointlike = None,
) -> Step:
    """Install pnpm package manager.

    Parameters
    ----------
    name
        The name of the step to display on GitHub.
    version
        The branch, ref, or SHA of the action's repository to use.
    pnpm_version
        Version of pnpm to install.
    dest
        Where to store pnpm files.
    run_install
        If specified, run ``pnpm install``.
    cache
        Whether to cache the pnpm store directory.
    cache_dependency_path
        File path to the pnpm lockfile for caching.
    package_json_file
        File path to the package.json to read ``packageManager`` configuration.
    standalone
        When true, installs @pnpm/exe to allow pnpm usage without Node.js.
    args
        The inputs for a Docker container which are passed to the container's entrypoint.
        This is a subkey of the ``with`` key of the generated step.
    entrypoint
        Overrides the Docker ENTRYPOINT in the action's Dockerfile or sets one if it was not
        specified. Accepts a single string defining the executable to run (note that this is
        different from Docker's ENTRYPOINT instruction which has both a shell and exec form).
        This is a subkey of the ``with`` key of the generated step.
    condition
        A boolean expression which must be met for the step to run. Note that this represents
        the ``if`` key in the actual YAML file.
    id
        A unique identifier for the step which can be referenced in expressions.
    env
        Used to specify environment variables for the step.
    continue_on_error
        Prevents the job from failing if this step fails.
    timeout_minutes
        The maximum number of minutes to let the step run before GitHub automatically
        cancels it (defaults to 360 if not specified).

    Returns
    -------
    Step
        The generated setup-pnpm step.

    See Also
    --------
    GitHub repository: https://github.com/pnpm/action-setup
    """
    options: dict[str, object] = {
        'version': pnpm_version,
        'dest': dest,
        'run_install': run_install,
        'cache': cache,
        'cache_dependency_path': cache_dependency_path,
        'package_json_file': package_json_file,
        'standalone': standalone,
    }

    options = {key: value for key, value in options.items() if value is not None}

    if name is None:
        name = 'Setup pnpm'

    return action(
        name,
        'pnpm/action-setup',
        ref=version,
        with_opts=options or None,
        args=args,
        entrypoint=entrypoint,
        condition=condition,
        id=id,
        env=env,
        continue_on_error=continue_on_error,
        timeout_minutes=timeout_minutes,
    )



from __future__ import annotations
from yamloom.actions.utils import validate_choice

from dataclasses import dataclass
from typing import TYPE_CHECKING

from ...expressions import context, StringExpression
from ..._yamloom import Step
from ..._yamloom import action
from ..types import (
    Oboollike,
    Oboolstr,
    Ointlike,
    Ostr,
    Ostrlike,
    StringOrBoolLike,
    StringLike,
)

if TYPE_CHECKING:
    from collections.abc import Mapping

__all__ = ['create_pull_request', 'CreatePullRequestOutput']


@dataclass(frozen=True)
class CreatePullRequestOutput:
    """Typed access to outputs produced by the create_pull_request step.

    Parameters
    ----------
    id
        The ``id`` of the create_pull_request step whose outputs should be
        referenced. This should match the ``id`` passed to
        :func:`create_pull_request`.

    Attributes
    ----------
    pull_request_number
        The pull request number.
    pull_request_url
        The URL of the pull request.
    pull_request_operation
        The pull request operation performed by the action, ``created``,
        ``updated``, ``closed`` or ``none``.
    pull_request_head_sha
        The commit SHA of the pull request branch.
    pull_request_branch
        The branch name of the pull request.
    pull_request_commits_verified
        Whether GitHub considers the signature of the branch's commits to be
        verified; ``true`` or ``false``.

    See Also
    --------
    GitHub repository: https://github.com/peter-evans/create-pull-request
    """

    id: str

    @property
    def pull_request_number(self) -> StringExpression:
        return context.steps[self.id].outputs['pull-request-number']

    @property
    def pull_request_url(self) -> StringExpression:
        return context.steps[self.id].outputs['pull-request-url']

    @property
    def pull_request_operation(self) -> StringExpression:
        return context.steps[self.id].outputs['pull-request-operation']

    @property
    def pull_request_head_sha(self) -> StringExpression:
        return context.steps[self.id].outputs['pull-request-head-sha']

    @property
    def pull_request_branch(self) -> StringExpression:
        return context.steps[self.id].outputs['pull-request-branch']

    @property
    def pull_request_commits_verified(self) -> StringExpression:
        return context.steps[self.id].outputs['pull-request-commits-verified']


def create_pull_request(
    *,
    name: Ostrlike = None,
    version: str = 'v8',
    token: Ostrlike = None,
    branch_token: Ostrlike = None,
    path: Ostrlike = None,
    add_paths: Ostrlike = None,
    commit_message: Ostrlike = None,
    committer: Ostrlike = None,
    author: Ostrlike = None,
    signoff: Oboollike = None,
    branch: Ostrlike = None,
    delete_branch: Oboollike = None,
    branch_suffix: Ostrlike = None,
    base: Ostrlike = None,
    push_to_fork: Ostrlike = None,
    sign_commits: Oboollike = None,
    title: Ostrlike = None,
    body: Ostrlike = None,
    body_path: Ostrlike = None,
    labels: Ostrlike = None,
    assignees: Ostrlike = None,
    reviewers: Ostrlike = None,
    team_reviewers: Ostrlike = None,
    milestone: Ointlike = None,
    draft: StringOrBoolLike | None = None,
    maintainer_can_modify: Oboollike = None,
    args: Ostrlike = None,
    entrypoint: Ostrlike = None,
    condition: Oboolstr = None,
    id: Ostr = None,  # noqa: A002
    env: Mapping[str, StringLike] | None = None,
    continue_on_error: Oboollike = None,
    timeout_minutes: Ointlike = None,
) -> Step:
    """Creates a pull request for changes to your repository in the actions workspace.

    Parameters
    ----------
    name
        The name of the step to display on GitHub.
    version
        The branch, ref, or SHA of the action's repository to use.
    token
        The token that the action will use to create and update the pull request.
    branch_token
        The token that the action will use to create and update the branch.
        Defaults to the value of ``token``.
    path
        Relative path under ``$GITHUB_WORKSPACE`` to the repository. Defaults to
        ``$GITHUB_WORKSPACE``.
    add_paths
        A comma or newline-separated list of file paths to commit. Paths should
        follow git's pathspec syntax. Defaults to adding all new and modified
        files.
    commit_message
        The message to use when committing changes.
    committer
        The committer name and email address in the format
        ``Display Name <email@address.com>``. Defaults to the GitHub Actions bot
        user.
    author
        The author name and email address in the format
        ``Display Name <email@address.com>``. Defaults to the user who triggered
        the workflow run.
    signoff
        Add ``Signed-off-by`` line by the committer at the end of the commit log
        message.
    branch
        The pull request branch name.
    delete_branch
        Delete ``branch`` if it doesn't have an active pull request associated
        with it.
    branch_suffix
        The branch suffix type when using the alternative branching strategy.
        Valid values are ``random``, ``timestamp`` and ``short-commit-hash``.
    base
        The pull request base branch. Defaults to the branch checked out in the
        workflow.
    push_to_fork
        A fork of the checked-out parent repository to which the pull request
        branch will be pushed (e.g. ``owner/repo-fork``). The pull request will be
        created to merge the fork's branch into the parent's base.
    sign_commits
        Sign commits as ``github-actions[bot]`` when using ``GITHUB_TOKEN``, or
        your own bot when using GitHub App tokens.
    title
        The title of the pull request.
    body
        The body of the pull request.
    body_path
        The path to a file containing the pull request body. Takes precedence
        over ``body``.
    labels
        A comma or newline separated list of labels.
    assignees
        A comma or newline separated list of assignees (GitHub usernames).
    reviewers
        A comma or newline separated list of reviewers (GitHub usernames) to
        request a review from.
    team_reviewers
        A comma or newline separated list of GitHub teams to request a review
        from. A ``repo`` scoped Personal Access Token (PAT) or equivalent GitHub
        App permissions may be required.
    milestone
        The number of the milestone to associate the pull request with.
    draft
        Create a draft pull request. Valid values are ``true`` (only on create),
        ``always-true`` (on create and update), and ``false``.
    maintainer_can_modify
        Indicates whether maintainers can modify the pull request.
    args
        The inputs for a Docker container which are passed to the container's entrypoint.
        This is a subkey of the ``with`` key of the generated step.
    entrypoint
        Overrides the Docker ENTRYPOINT in the action's Dockerfile or sets one if it was not
        specified. Accepts a single string defining the executable to run (note that this is
        different from Docker's ENTRYPOINT instruction which has both a shell and exec form).
        This is a subkey of the ``with`` key of the generated step.
    condition
        A boolean expression which must be met for the step to run. Note that this represents
        the ``if`` key in the actual YAML file.
    id
        A unique identifier for the step which can be referenced in expressions.
    env
        Used to specify environment variables for the step.
    continue_on_error
        Prevents the job from failing if this step fails.
    timeout_minutes
        The maximum number of minutes to let the step run before GitHub automatically
        cancels it (defaults to 360 if not specified).

    Returns
    -------
    Step
        The generated create pull request step.

    See Also
    --------
    GitHub repository: https://github.com/peter-evans/create-pull-request
    """
    options: dict[str, object] = {
        'token': token,
        'branch-token': branch_token,
        'path': path,
        'add-paths': add_paths,
        'commit-message': commit_message,
        'committer': committer,
        'author': author,
        'signoff': signoff,
        'branch': branch,
        'delete-branch': delete_branch,
        'branch-suffix': validate_choice(
            'branch_suffix', branch_suffix, ['random', 'timestamp', 'short-commit-hash']
        ),
        'base': base,
        'push-to-fork': push_to_fork,
        'sign-commits': sign_commits,
        'title': title,
        'body': body,
        'body-path': body_path,
        'labels': labels,
        'assignees': assignees,
        'reviewers': reviewers,
        'team-reviewers': team_reviewers,
        'milestone': milestone,
        'draft': draft,
        'maintainer-can-modify': maintainer_can_modify,
    }
    options = {key: value for key, value in options.items() if value is not None}

    if name is None:
        name = 'Create Pull Request'

    return action(
        name,
        'peter-evans/create-pull-request',
        ref=version,
        with_opts=options or None,
        args=args,
        entrypoint=entrypoint,
        condition=condition,
        id=id,
        env=env,
        continue_on_error=continue_on_error,
        timeout_minutes=timeout_minutes,
    )



from __future__ import annotations
from yamloom.actions.utils import validate_choice

from dataclasses import dataclass
from typing import TYPE_CHECKING

from ...expressions import context, StringExpression
from ..._yamloom import Step
from ..._yamloom import action
from ..types import (
    Oboollike,
    Oboolstr,
    Ointlike,
    Ostr,
    Ostrlike,
    StringLike,
)

if TYPE_CHECKING:
    from collections.abc import Mapping

__all__ = ['setup_java', 'SetupJavaOutput']


@dataclass(frozen=True)
class SetupJavaOutput:
    """Typed access to outputs produced by the setup_java step.

    Parameters
    ----------
    id
        The ``id`` of the setup_java step whose outputs should be referenced.
        This should match the ``id`` passed to :func:`setup_java`.

    Attributes
    ----------
    distribution
        Distribution of Java that has been installed.
    version
        Actual version of the Java environment that has been installed.
    path
        Path to where the Java environment has been installed (same as
        ``$JAVA_HOME``).
    cache_hit
        A boolean value to indicate if a cache was hit.

    See Also
    --------
    GitHub repository: https://github.com/actions/setup-java
    """

    id: str

    @property
    def distribution(self) -> StringExpression:
        return context.steps[self.id].outputs.distribution

    @property
    def version(self) -> StringExpression:
        return context.steps[self.id].outputs.version

    @property
    def path(self) -> StringExpression:
        return context.steps[self.id].outputs.path

    @property
    def cache_hit(self) -> StringExpression:
        return context.steps[self.id].outputs['cache-hit']


def setup_java(
    *,
    name: Ostrlike = None,
    version: str = 'v5',
    java_version: Ostrlike = None,
    java_version_file: Ostrlike = None,
    distribution: Ostrlike = None,
    java_package: Ostrlike = None,
    check_latest: Oboollike = None,
    architecture: Ostrlike = None,
    jdk_file: Ostrlike = None,
    cache: Ostrlike = None,
    cache_dependency_path: Ostrlike = None,
    overwrite_settings: Oboollike = None,
    server_id: Ostrlike = None,
    server_username: Ostrlike = None,
    server_password: Ostrlike = None,
    settings_path: Ostrlike = None,
    gpg_private_key: Ostrlike = None,
    gpg_passphrase: Ostrlike = None,
    mvn_toolchain_id: Ostrlike = None,
    mvn_toolchain_vendor: Ostrlike = None,
    args: Ostrlike = None,
    entrypoint: Ostrlike = None,
    condition: Oboolstr = None,
    id: Ostr = None,  # noqa: A002
    env: Mapping[str, StringLike] | None = None,
    continue_on_error: Oboollike = None,
    timeout_minutes: Ointlike = None,
) -> Step:
    """Set up a specific version of the Java JDK and add tools to PATH.

    Parameters
    ----------
    name
        The name of the step to display on GitHub.
    version
        The branch, ref, or SHA of the action's repository to use.
    java_version
        The Java version to set up. Takes a whole or semver Java version.
    java_version_file
        Path to the ``.java-version`` file.
    distribution
        Java distribution (required).
    java_package
        The package type (``jdk``, ``jre``, ``jdk+fx``, ``jre+fx``).
    architecture
        The architecture of the package.
    jdk_file
        Path to where the compressed JDK is located.
    check_latest
        Check for the latest available version that satisfies the version spec.
    server_id
        ID of the distributionManagement repository in the pom.xml file.
    server_username
        Environment variable name for the username for authentication to the
        Apache Maven repository.
    server_password
        Environment variable name for password or token for authentication to
        the Apache Maven repository.
    settings_path
        Path to where the settings.xml file will be written.
    overwrite_settings
        Overwrite the settings.xml file if it exists.
    gpg_private_key
        GPG private key to import.
    gpg_passphrase
        Environment variable name for the GPG private key passphrase.
    cache
        Name of the build platform to cache dependencies (``maven``, ``gradle``,
        ``sbt``).
    cache_dependency_path
        Path to a dependency file such as pom.xml, build.gradle, build.sbt, etc.
        Supports wildcards or a list of file names for caching multiple
        dependencies.
    mvn_toolchain_id
        Name of Maven Toolchain ID if the default name is not wanted.
    mvn_toolchain_vendor
        Name of Maven Toolchain Vendor if the default name is not wanted.
    args
        The inputs for a Docker container which are passed to the container's entrypoint.
        This is a subkey of the ``with`` key of the generated step.
    entrypoint
        Overrides the Docker ENTRYPOINT in the action's Dockerfile or sets one if it was not
        specified. Accepts a single string defining the executable to run (note that this is
        different from Docker's ENTRYPOINT instruction which has both a shell and exec form).
        This is a subkey of the ``with`` key of the generated step.
    condition
        A boolean expression which must be met for the step to run. Note that this represents
        the ``if`` key in the actual YAML file.
    id
        A unique identifier for the step which can be referenced in expressions.
    env
        Used to specify environment variables for the step.
    continue_on_error
        Prevents the job from failing if this step fails.
    timeout_minutes
        The maximum number of minutes to let the step run before GitHub automatically
        cancels it (defaults to 360 if not specified).

    Returns
    -------
    Step
        The generated setup-java step.

    See Also
    --------
    GitHub repository: https://github.com/actions/setup-java
    """
    options: dict[str, object] = {
        'java-version': java_version,
        'java-version-file': java_version_file,
        'distribution': distribution,
        'java-package': validate_choice(
            'java_package', java_package, ['jdk', 'jre', 'jdk+fx', 'jre+fx']
        ),
        'check-latest': check_latest,
        'architecture': validate_choice(
            'architecture', architecture, ['x86', 'x64', 'armv7', 'aarch64', 'ppc64le']
        ),
        'jdkFile': jdk_file,
        'cache': validate_choice('cache', cache, ['maven', 'gradle', 'sbt']),
        'cache-dependency-path': cache_dependency_path,
        'overwrite-settings': overwrite_settings,
        'server-id': server_id,
        'server-username': server_username,
        'server-password': server_password,
        'settings-path': settings_path,
        'gpg-private-key': gpg_private_key,
        'gpg-passphrase': gpg_passphrase,
        'mvn-toolchain-id': mvn_toolchain_id,
        'mvn-toolchain-vendor': mvn_toolchain_vendor,
    }
    options = {key: value for key, value in options.items() if value is not None}

    if name is None:
        name = 'Setup Java'

    return action(
        name,
        'actions/setup-java',
        ref=version,
        with_opts=options or None,
        args=args,
        entrypoint=entrypoint,
        condition=condition,
        id=id,
        env=env,
        continue_on_error=continue_on_error,
        timeout_minutes=timeout_minutes,
    )



from __future__ import annotations
from yamloom.actions.utils import check_string

from dataclasses import dataclass
from typing import TYPE_CHECKING

from ...expressions import context, StringExpression
from ..._yamloom import Step
from ..._yamloom import action
from ..types import (
    Oboollike,
    Oboolstr,
    Ointlike,
    Ostr,
    Ostrlike,
    StringLike,
)

if TYPE_CHECKING:
    from collections.abc import Mapping

__all__ = ['checkout', 'CheckoutOutput']


@dataclass(frozen=True)
class CheckoutOutput:
    """Typed access to outputs produced by the checkout step.

    Parameters
    ----------
    id
        The ``id`` of the checkout step whose outputs should be referenced.
        This should match the ``id`` passed to :func:`checkout`.

    Attributes
    ----------
    ref
        The branch, tag or SHA that was checked out.
    commit
        The commit SHA that was checked out.

    See Also
    --------
    GitHub repository: https://github.com/actions/checkout
    """

    id: str

    @property
    def ref(self) -> StringExpression:
        return context.steps[self.id].outputs.ref

    @property
    def commit(self) -> StringExpression:
        return context.steps[self.id].outputs.commit


def checkout(
    *,
    name: Ostrlike = None,
    version: str = 'v6',
    repository: Ostrlike = None,
    ref: Ostrlike = None,
    token: Ostrlike = None,
    ssh_key: Ostrlike = None,
    ssh_known_hosts: Ostrlike = None,
    ssh_strict: Oboollike = None,
    ssh_user: Ostrlike = None,
    persist_credentials: Oboollike = None,
    path: Ostrlike = None,
    clean: Oboollike = None,
    filter: Ostrlike = None,  # noqa: A002
    sparse_checkout: Ostrlike = None,
    sparse_checkout_cone_mode: Oboollike = None,
    fetch_depth: Ointlike = None,
    fetch_tags: Oboollike = None,
    show_progress: Oboollike = None,
    lfs: Oboollike = None,
    submodules: Oboollike = None,
    get_safe_directory: Oboollike = None,
    github_server_url: Ostrlike = None,
    args: Ostrlike = None,
    entrypoint: Ostrlike = None,
    condition: Oboolstr = None,
    id: Ostr = None,  # noqa: A002
    env: Mapping[str, StringLike] | None = None,
    continue_on_error: Oboollike = None,
    timeout_minutes: Ointlike = None,
) -> Step:
    """Checkout a Git repository at a particular version.

    Parameters
    ----------
    name
        The name of the step to display on GitHub.
    version
        The branch, ref, or SHA of the action's repository to use.
    repository
        Repository name with owner. For example, ``actions/checkout``.
    ref
        The branch, tag or SHA to checkout. When checking out the repository that
        triggered a workflow, this defaults to the reference or SHA for that event.
        Otherwise, uses the default branch.
    token
        Personal access token (PAT) used to fetch the repository. The PAT is configured
        with the local git config, which enables your scripts to run authenticated git
        commands. The post-job step removes the PAT.

        We recommend using a service account with the least permissions necessary.
        Also when generating a new PAT, select the least scopes necessary.
    ssh_key
        SSH key used to fetch the repository. The SSH key is configured with the local
        git config, which enables your scripts to run authenticated git commands.
        The post-job step removes the SSH key.

        We recommend using a service account with the least permissions necessary.
    ssh_known_hosts
        Known hosts in addition to the user and global host key database. The public
        SSH keys for a host may be obtained using the utility ``ssh-keyscan``. For example,
        ``ssh-keyscan github.com``. The public key for github.com is always implicitly added.
    ssh_strict
        Whether to perform strict host key checking. When true, adds the options
        ``StrictHostKeyChecking=yes`` and ``CheckHostIP=no`` to the SSH command line.
        Use ``ssh_known_hosts`` to configure additional hosts.
    ssh_user
        The user to use when connecting to the remote SSH host. By default ``git`` is used.
    persist_credentials
        Whether to configure the token or SSH key with the local git config.
    path
        Relative path under ``$GITHUB_WORKSPACE`` to place the repository.
    clean
        Whether to execute ``git clean -ffdx && git reset --hard HEAD`` before fetching.
    filter
        Partially clone against a given filter. Overrides ``sparse_checkout`` if set.
    sparse_checkout
        Do a sparse checkout on given patterns. Each pattern should be separated with
        new lines.
    sparse_checkout_cone_mode
        Specifies whether to use cone-mode when doing a sparse checkout.
    fetch_depth
        Number of commits to fetch. ``0`` indicates all history for all branches and tags.
    fetch_tags
        Whether to fetch tags, even if ``fetch_depth > 0``.
    show_progress
        Whether to show progress status output when fetching.
    lfs
        Whether to download Git-LFS files.
    submodules
        Whether to checkout submodules: ``true`` to checkout submodules or ``recursive`` to
        recursively checkout submodules.

        When ``ssh_key`` is not provided, SSH URLs beginning with ``git@github.com:``
        are converted to HTTPS.
    get_safe_directory
        Add repository path as ``safe.directory`` for Git global config by running
        ``git config --global --add safe.directory <path>``.
    github_server_url
        The base URL for the GitHub instance that you are trying to clone from. Uses
        environment defaults to fetch from the same instance that the workflow is running
        from unless specified.
    args
        The inputs for a Docker container which are passed to the container's entrypoint.
        This is a subkey of the ``with`` key of the generated step.
    entrypoint
        Overrides the Docker ENTRYPOINT in the action's Dockerfile or sets one if it was not
        specified. Accepts a single string defining the executable to run (note that this is
        different from Docker's ENTRYPOINT instruction which has both a shell and exec form).
        This is a subkey of the ``with`` key of the generated step.
    condition
        A boolean expression which must be met for the step to run. Note that this represents
        the ``if`` key in the actual YAML file.
    id
        A unique identifier for the step which can be referenced in expressions.
    env
        Used to specify environment variables for the step.
    continue_on_error
        Prevents the job from failing if this step fails.
    timeout_minutes
        The maximum number of minutes to let the step run before GitHub automatically
        cancels it (defaults to 360 if not specified).

    Returns
    -------
    Step
        The generated checkout step.

    See Also
    --------
    GitHub repository: https://github.com/actions/checkout
    """
    options: dict[str, object] = {
        'repository': repository,
        'ref': ref,
        'token': token,
        'ssh-key': ssh_key,
        'ssh-known-hosts': ssh_known_hosts,
        'ssh-strict': ssh_strict,
        'ssh-user': ssh_user,
        'persist-credentials': persist_credentials,
        'path': path,
        'clean': clean,
        'filter': filter,
        'sparse-checkout': sparse_checkout,
        'sparse-checkout-cone-mode': sparse_checkout_cone_mode,
        'fetch-depth': fetch_depth,
        'fetch-tags': fetch_tags,
        'show-progress': show_progress,
        'lfs': lfs,
        'submodules': submodules,
        'get-safe-directory': get_safe_directory,
        'github-server-url': github_server_url,
    }
    options = {key: value for key, value in options.items() if value is not None}

    if name is None:
        repository_str = check_string(options.get('repository'))
        if repository_str:
            name = f"Checkout '{repository_str}'"
        else:
            name = 'Checkout Repository'

    return action(
        name,
        'actions/checkout',
        ref=version,
        with_opts=options or None,
        args=args,
        entrypoint=entrypoint,
        condition=condition,
        id=id,
        env=env,
        continue_on_error=continue_on_error,
        timeout_minutes=timeout_minutes,
    )



from __future__ import annotations

from dataclasses import dataclass
from typing import TYPE_CHECKING

from ...expressions import context, StringExpression
from ..._yamloom import Step
from ..._yamloom import action
from ..types import (
    Oboollike,
    Oboolstr,
    Ointlike,
    Ostr,
    Ostrlike,
    StringLike,
)

if TYPE_CHECKING:
    from collections.abc import Mapping

__all__ = ['setup_bun', 'SetupBunOutput']


@dataclass(frozen=True)
class SetupBunOutput:
    """Typed access to outputs produced by the setup_bun step.

    Parameters
    ----------
    id
        The ``id`` of the setup_bun step whose outputs should be referenced.
        This should match the ``id`` passed to :func:`setup_bun`.

    Attributes
    ----------
    bun_version
        The version of Bun that was installed.
    bun_revision
        The revision of Bun that was installed.
    bun_path
        The path to the Bun executable.
    bun_download_url
        The URL from which Bun was downloaded.
    cache_hit
        Whether the Bun version was cached.

    See Also
    --------
    GitHub repository: https://github.com/oven-sh/setup-bun
    """

    id: str

    @property
    def bun_version(self) -> StringExpression:
        return context.steps[self.id].outputs['bun-version']

    @property
    def bun_revision(self) -> StringExpression:
        return context.steps[self.id].outputs['bun-revision']

    @property
    def bun_path(self) -> StringExpression:
        return context.steps[self.id].outputs['bun-path']

    @property
    def bun_download_url(self) -> StringExpression:
        return context.steps[self.id].outputs['bun-download-url']

    @property
    def cache_hit(self) -> StringExpression:
        return context.steps[self.id].outputs['cache-hit']


def setup_bun(
    *,
    name: Ostrlike = None,
    version: str = 'v2',
    bun_version: Ostrlike = None,
    bun_version_file: Ostrlike = None,
    bun_download_url: Ostrlike = None,
    registries: Ostrlike = None,
    no_cache: Oboollike = None,
    token: Ostrlike = None,
    args: Ostrlike = None,
    entrypoint: Ostrlike = None,
    condition: Oboolstr = None,
    id: Ostr = None,  # noqa: A002
    env: Mapping[str, StringLike] | None = None,
    continue_on_error: Oboollike = None,
    timeout_minutes: Ointlike = None,
) -> Step:
    """Download, install, and set up Bun on the PATH.

    Parameters
    ----------
    name
        The name of the step to display on GitHub.
    version
        The branch, ref, or SHA of the action's repository to use.
    bun_version
        The version of Bun to install (e.g. ``latest``, ``canary``, ``1.0.0``).
    bun_version_file
        The version of Bun to install from file (e.g. ``package.json``,
        ``.bun-version``).
    bun_download_url
        Override the URL to download Bun from.
    registries
        List of package registries with authentication support.
    no_cache
        Disable caching of the Bun executable.
    token
        Personal access token (PAT) used to fetch tags from oven-sh/bun.
    args
        The inputs for a Docker container which are passed to the container's entrypoint.
        This is a subkey of the ``with`` key of the generated step.
    entrypoint
        Overrides the Docker ENTRYPOINT in the action's Dockerfile or sets one if it was not
        specified. Accepts a single string defining the executable to run (note that this is
        different from Docker's ENTRYPOINT instruction which has both a shell and exec form).
        This is a subkey of the ``with`` key of the generated step.
    condition
        A boolean expression which must be met for the step to run. Note that this represents
        the ``if`` key in the actual YAML file.
    id
        A unique identifier for the step which can be referenced in expressions.
    env
        Used to specify environment variables for the step.
    continue_on_error
        Prevents the job from failing if this step fails.
    timeout_minutes
        The maximum number of minutes to let the step run before GitHub automatically
        cancels it (defaults to 360 if not specified).

    Returns
    -------
    Step
        The generated setup-bun step.

    See Also
    --------
    GitHub repository: https://github.com/oven-sh/setup-bun
    """
    options: dict[str, object] = {
        'bun-version': bun_version,
        'bun-version-file': bun_version_file,
        'bun-download-url': bun_download_url,
        'registries': registries,
        'no-cache': no_cache,
        'token': token,
    }
    options = {key: value for key, value in options.items() if value is not None}

    if name is None:
        name = 'Setup Bun'

    return action(
        name,
        'oven-sh/setup-bun',
        ref=version,
        with_opts=options or None,
        args=args,
        entrypoint=entrypoint,
        condition=condition,
        id=id,
        env=env,
        continue_on_error=continue_on_error,
        timeout_minutes=timeout_minutes,
    )



from __future__ import annotations
from yamloom.actions.utils import validate_choice, check_string

from dataclasses import dataclass
from typing import TYPE_CHECKING

from ...expressions import context, StringExpression
from ..._yamloom import Step
from ..._yamloom import action
from ..types import (
    Oboollike,
    Oboolstr,
    Ointlike,
    Ostr,
    Ostrlike,
    StringLike,
)

if TYPE_CHECKING:
    from collections.abc import Mapping

__all__ = ['setup_mpi', 'SetupMpiOutput']


@dataclass(frozen=True)
class SetupMpiOutput:
    """Typed access to outputs produced by the setup_mpi step.

    Parameters
    ----------
    id
        The ``id`` of the setup_mpi step whose outputs should be referenced.
        This should match the ``id`` passed to :func:`setup_mpi`.

    Attributes
    ----------
    mpi
        The installed MPI implementation name.

    See Also
    --------
    GitHub repository: https://github.com/mpi4py/setup-mpi
    """

    id: str

    @property
    def mpi(self) -> StringExpression:
        return context.steps[self.id].outputs.mpi


def setup_mpi(
    *,
    name: Ostrlike = None,
    version: str = 'v1',
    mpi: Ostrlike = None,
    args: Ostrlike = None,
    entrypoint: Ostrlike = None,
    condition: Oboolstr = None,
    id: Ostr = None,  # noqa: A002
    env: Mapping[str, StringLike] | None = None,
    continue_on_error: Oboollike = None,
    timeout_minutes: Ointlike = None,
) -> Step:
    """Set up a specific MPI implementation.

    Parameters
    ----------
    name
        The name of the step to display on GitHub.
    version
        The branch, ref, or SHA of the action's repository to use.
    mpi
        MPI implementation name.
    args
        The inputs for a Docker container which are passed to the container's entrypoint.
        This is a subkey of the ``with`` key of the generated step.
    entrypoint
        Overrides the Docker ENTRYPOINT in the action's Dockerfile or sets one if it was not
        specified. Accepts a single string defining the executable to run (note that this is
        different from Docker's ENTRYPOINT instruction which has both a shell and exec form).
        This is a subkey of the ``with`` key of the generated step.
    condition
        A boolean expression which must be met for the step to run. Note that this represents
        the ``if`` key in the actual YAML file.
    id
        A unique identifier for the step which can be referenced in expressions.
    env
        Used to specify environment variables for the step.
    continue_on_error
        Prevents the job from failing if this step fails.
    timeout_minutes
        The maximum number of minutes to let the step run before GitHub automatically
        cancels it (defaults to 360 if not specified).

    Returns
    -------
    Step
        The generated setup-mpi step.

    See Also
    --------
    GitHub repository: https://github.com/mpi4py/setup-mpi
    """
    options: dict[str, object] = {
        'mpi': validate_choice('mpi', mpi, ['mpich', 'openmpi', 'intelmpi', 'msmpi']),
    }

    options = {key: value for key, value in options.items() if value is not None}

    mpi_names = {
        'mpich': 'MPICH',
        'openmpi': 'Open MPI',
        'intelmpi': 'Intel MPI',
        'msmpi': 'Microsoft MPI',
    }

    if name is None:
        mpi_str = check_string(options.get('mpi'))
        if mpi_str:
            name = f'Setup {mpi_names[mpi_str]}'
        else:
            name = 'Setup MPI'

    return action(
        name,
        'mpi4py/setup-mpi',
        ref=version,
        with_opts=options or None,
        args=args,
        entrypoint=entrypoint,
        condition=condition,
        id=id,
        env=env,
        continue_on_error=continue_on_error,
        timeout_minutes=timeout_minutes,
    )



from __future__ import annotations
from yamloom.actions.utils import validate_choice, check_string

from dataclasses import dataclass
from typing import TYPE_CHECKING

from ...expressions import context, StringExpression
from ..._yamloom import Step
from ..._yamloom import action
from ..types import (
    Oboollike,
    Oboolstr,
    Ointlike,
    Ostr,
    Ostrlike,
    StringLike,
)

if TYPE_CHECKING:
    from collections.abc import Mapping

WARN_RETENTION_DAYS: int = 90
MAX_COMPRESSION_LEVEL: int = 9

__all__ = [
    'download_artifact',
    'upload_artifact',
    'upload_artifact_merge',
    'DownloadArtifactOutput',
    'UploadArtifactOutput',
    'UploadArtifactMergeOutput',
]


@dataclass(frozen=True)
class UploadArtifactOutput:
    """Typed access to outputs produced by the upload_artifact step.

    Parameters
    ----------
    id
        The ``id`` of the upload_artifact step whose outputs should be referenced.
        This should match the ``id`` passed to :func:`upload_artifact`.

    Attributes
    ----------
    artifact_id
        A unique identifier for the artifact that was just uploaded. Empty if the
        artifact upload failed.
    artifact_url
        A download URL for the artifact that was just uploaded. Empty if the artifact
        upload failed. This URL is valid only while the artifact, workflow run, and
        repository exist.
    artifact_digest
        SHA-256 digest for the artifact that was just uploaded. Empty if the artifact
        upload failed.

    See Also
    --------
    GitHub repository: https://github.com/actions/upload-artifact
    """

    id: str

    @property
    def artifact_id(self) -> StringExpression:
        return context.steps[self.id].outputs['artifact-id']

    @property
    def artifact_url(self) -> StringExpression:
        return context.steps[self.id].outputs['artifact-url']

    @property
    def artifact_digest(self) -> StringExpression:
        return context.steps[self.id].outputs['artifact-digest']


@dataclass(frozen=True)
class UploadArtifactMergeOutput:
    """Typed access to outputs produced by the upload_artifact_merge step.

    Parameters
    ----------
    id
        The ``id`` of the upload_artifact_merge step whose outputs should be referenced.
        This should match the ``id`` passed to :func:`upload_artifact_merge`.

    Attributes
    ----------
    artifact_id
        A unique identifier for the artifact that was just uploaded. Empty if the
        artifact upload failed.
    artifact_url
        A download URL for the artifact that was just uploaded. Empty if the artifact
        upload failed. This URL is valid only while the artifact, workflow run, and
        repository exist.
    artifact_digest
        SHA-256 digest for the artifact that was just uploaded. Empty if the artifact
        upload failed.

    See Also
    --------
    GitHub repository: https://github.com/actions/upload-artifact
    """

    id: str

    @property
    def artifact_id(self) -> StringExpression:
        return context.steps[self.id].outputs['artifact-id']

    @property
    def artifact_url(self) -> StringExpression:
        return context.steps[self.id].outputs['artifact-url']

    @property
    def artifact_digest(self) -> StringExpression:
        return context.steps[self.id].outputs['artifact-digest']


@dataclass(frozen=True)
class DownloadArtifactOutput:
    """Typed access to outputs produced by the download_artifact step.

    Parameters
    ----------
    id
        The ``id`` of the download_artifact step whose outputs should be referenced.
        This should match the ``id`` passed to :func:`download_artifact`.

    Attributes
    ----------
    download_path
        Path of artifact download.

    See Also
    --------
    GitHub repository: https://github.com/actions/download-artifact
    """

    id: str

    @property
    def download_path(self) -> StringExpression:
        return context.steps[self.id].outputs['download-path']


def upload_artifact(
    *,
    path: StringLike,
    name: Ostrlike = None,
    version: str = 'v6',
    artifact_name: Ostrlike = None,
    if_no_files_found: Ostrlike = None,
    retention_days: Ointlike = None,
    compression_level: Ointlike = None,
    overwrite: Oboollike = None,
    include_hidden_files: Oboollike = None,
    args: Ostrlike = None,
    entrypoint: Ostrlike = None,
    condition: Oboolstr = None,
    id: Ostr = None,  # noqa: A002
    env: Mapping[str, StringLike] | None = None,
    continue_on_error: Oboollike = None,
    timeout_minutes: Ointlike = None,
) -> Step:
    """Upload a build artifact that can be used by subsequent workflow steps.

    Parameters
    ----------
    path
        A file, directory or wildcard pattern that describes what to upload.
    name
        The name of the step to display on GitHub.
    version
        The branch, ref, or SHA of the action's repository to use.
    artifact_name
        Artifact name.
    if_no_files_found
        The desired behavior if no files are found using the provided path. Available
        options are ``warn``, ``error``, and ``ignore``.
    retention_days
        Duration after which artifact will expire in days. ``0`` means using default
        retention. Minimum 1 day. Maximum 90 days unless changed from the repository
        settings page.
    compression_level
        The level of compression for Zlib to be applied to the artifact archive.
        The value can range from 0 to 9, where 0 is no compression and 9 is best
        compression. Higher levels take longer to complete.
    overwrite
        If true, an artifact with a matching name will be deleted before a new one
        is uploaded. If false, the action will fail if an artifact for the given name
        already exists. Does not fail if the artifact does not exist.
    include_hidden_files
        If true, hidden files will be included in the artifact. If false, hidden files
        will be excluded from the artifact.
    args
        The inputs for a Docker container which are passed to the container's entrypoint.
        This is a subkey of the ``with`` key of the generated step.
    entrypoint
        Overrides the Docker ENTRYPOINT in the action's Dockerfile or sets one if it was not
        specified. Accepts a single string defining the executable to run (note that this is
        different from Docker's ENTRYPOINT instruction which has both a shell and exec form).
        This is a subkey of the ``with`` key of the generated step.
    condition
        A boolean expression which must be met for the step to run. Note that this represents
        the ``if`` key in the actual YAML file.
    id
        A unique identifier for the step which can be referenced in expressions.
    env
        Used to specify environment variables for the step.
    continue_on_error
        Prevents the job from failing if this step fails.
    timeout_minutes
        The maximum number of minutes to let the step run before GitHub automatically
        cancels it (defaults to 360 if not specified).

    Returns
    -------
    Step
        The generated upload artifact step.

    See Also
    --------
    GitHub repository: https://github.com/actions/upload-artifact
    """
    options: dict[str, object] = {
        'path': path,
        'name': artifact_name,
        'if_no_files_found': validate_choice(
            'if_no_files_found', if_no_files_found, ['warn', 'error', 'ignore']
        ),
        'retention-days': retention_days,
        'compression-level': compression_level,
        'overwrite': overwrite,
        'include-hidden-files': include_hidden_files,
    }

    if retention_days is not None:
        if isinstance(retention_days, int) and not isinstance(retention_days, bool):
            if retention_days < 1:
                msg = 'retention days must be > 0'
                raise ValueError(msg)
            if retention_days > WARN_RETENTION_DAYS:
                print(
                    f'Warning: retention days should be <= {WARN_RETENTION_DAYS} unless a higher limit is made in the repository settings!'
                )
        options['retention-days'] = retention_days

    if compression_level is not None:
        if (
            isinstance(compression_level, int)
            and not isinstance(compression_level, bool)
        ) and (compression_level < 0 or compression_level > MAX_COMPRESSION_LEVEL):
            msg = f'compression level must be in the range 0-{MAX_COMPRESSION_LEVEL}'
            raise ValueError(msg)
        options['compression-level'] = compression_level

    options = {key: value for key, value in options.items() if value is not None}

    if name is None:
        artifact_str = check_string(options.get('artifact_name'))
        if artifact_str:
            name = f'Upload {artifact_str}'
        else:
            name = 'Upload Artifact'

    return action(
        name,
        'actions/upload-artifact',
        ref=version,
        with_opts=options or None,
        args=args,
        entrypoint=entrypoint,
        condition=condition,
        id=id,
        env=env,
        continue_on_error=continue_on_error,
        timeout_minutes=timeout_minutes,
    )


def upload_artifact_merge(
    *,
    pattern: Ostrlike = None,
    name: Ostrlike = None,
    version: str = 'v6',
    artifact_name: Ostrlike = None,
    separate_directories: Oboollike = None,
    delete_merged: Oboollike = None,
    retention_days: Ointlike = None,
    compression_level: Ointlike = None,
    include_hidden_files: Oboollike = None,
    args: Ostrlike = None,
    entrypoint: Ostrlike = None,
    condition: Oboolstr = None,
    id: Ostr = None,  # noqa: A002
    env: Mapping[str, StringLike] | None = None,
    continue_on_error: Oboollike = None,
    timeout_minutes: Ointlike = None,
) -> Step:
    """Merge one or more build artifacts.

    Parameters
    ----------
    pattern
        A glob pattern matching the artifact names that should be merged.
    name
        The name of the step to display on GitHub.
    version
        The branch, ref, or SHA of the action's repository to use.
    artifact_name
        The name of the artifact that the artifacts will be merged into.
    separate_directories
        When multiple artifacts are matched, this changes the behavior of how they are
        merged in the archive. If true, the matched artifacts are extracted into
        individual named directories within the specified path. If false, the matched
        artifacts are combined in the same directory.
    delete_merged
        If true, the artifacts that were merged will be deleted. If false, the artifacts
        will still exist.
    retention_days
        Duration after which artifact will expire in days. ``0`` means using default
        retention. Minimum 1 day. Maximum 90 days unless changed from the repository
        settings page.
    compression_level
        The level of compression for Zlib to be applied to the artifact archive.
        The value can range from 0 to 9, where 0 is no compression and 9 is best
        compression. Higher levels take longer to complete.
    include_hidden_files
        If true, hidden files will be included in the merged artifact. If false, hidden
        files will be excluded from the merged artifact.
    args
        The inputs for a Docker container which are passed to the container's entrypoint.
        This is a subkey of the ``with`` key of the generated step.
    entrypoint
        Overrides the Docker ENTRYPOINT in the action's Dockerfile or sets one if it was not
        specified. Accepts a single string defining the executable to run (note that this is
        different from Docker's ENTRYPOINT instruction which has both a shell and exec form).
        This is a subkey of the ``with`` key of the generated step.
    condition
        A boolean expression which must be met for the step to run. Note that this represents
        the ``if`` key in the actual YAML file.
    id
        A unique identifier for the step which can be referenced in expressions.
    env
        Used to specify environment variables for the step.
    continue_on_error
        Prevents the job from failing if this step fails.
    timeout_minutes
        The maximum number of minutes to let the step run before GitHub automatically
        cancels it (defaults to 360 if not specified).

    Returns
    -------
    Step
        The generated upload merge step.

    See Also
    --------
    GitHub repository: https://github.com/actions/upload-artifact
    """
    options: dict[str, object] = {
        'name': artifact_name,
        'pattern': pattern,
        'separate-directories': separate_directories,
        'delete-merged': delete_merged,
        'retention-days': retention_days,
        'compression-level': compression_level,
        'include-hidden-files': include_hidden_files,
    }

    if retention_days is not None:
        if isinstance(retention_days, int) and not isinstance(retention_days, bool):
            if retention_days < 1:
                msg = 'retention days must be > 0'
                raise ValueError(msg)
            if retention_days > WARN_RETENTION_DAYS:
                print(
                    f'Warning: retention days should be <= {WARN_RETENTION_DAYS} unless a higher limit is made in the repository settings!'
                )
        options['retention-days'] = retention_days

    if compression_level is not None:
        if (
            isinstance(compression_level, int)
            and not isinstance(compression_level, bool)
        ) and (compression_level < 0 or compression_level > MAX_COMPRESSION_LEVEL):
            msg = f'compression level must be in the range 0-{MAX_COMPRESSION_LEVEL}'
            raise ValueError(msg)
        options['compression-level'] = compression_level

    options = {key: value for key, value in options.items() if value is not None}

    if name is None:
        artifact_str = check_string(options.get('artifact_name'))
        if artifact_str:
            name = f'Upload (merged) {artifact_str}'
        else:
            name = 'Upload (merged) Artifact'

    return action(
        name,
        'actions/upload-artifact/merge',
        ref=version,
        with_opts=options or None,
        args=args,
        entrypoint=entrypoint,
        condition=condition,
        id=id,
        env=env,
        continue_on_error=continue_on_error,
        timeout_minutes=timeout_minutes,
    )


def download_artifact(
    *,
    path: Ostrlike = None,
    name: Ostrlike = None,
    version: str = 'v7',
    artifact_name: Ostrlike = None,
    artifact_ids: list[StringLike] | None = None,
    pattern: Ostrlike = None,
    merge_multiple: Oboollike = None,
    github_token: Ostrlike = None,
    repository: Ostrlike = None,
    run_id: Ostrlike = None,
    args: Ostrlike = None,
    entrypoint: Ostrlike = None,
    condition: Oboolstr = None,
    id: Ostr = None,  # noqa: A002
    env: Mapping[str, StringLike] | None = None,
    continue_on_error: Oboollike = None,
    timeout_minutes: Ointlike = None,
) -> Step:
    """Download a build artifact that was previously uploaded in the workflow.

    Parameters
    ----------
    path
        Destination path. Supports basic tilde expansion. Defaults to ``$GITHUB_WORKSPACE``.
    name
        The name of the step to display on GitHub.
    version
        The branch, ref, or SHA of the action's repository to use.
    artifact_name
        Name of the artifact to download. If unspecified, all artifacts for the run
        are downloaded.
    artifact_ids
        IDs of the artifacts to download. Either ``artifact_ids`` or ``artifact_name``
        can be used, but not both.
    pattern
        A glob pattern matching the artifacts that should be downloaded. Ignored if
        ``artifact_name`` is specified.
    merge_multiple
        When multiple artifacts are matched, this changes the behavior of the
        destination directories. If true, the downloaded artifacts will be in the
        same directory specified by path. If false, the downloaded artifacts will be
        extracted into individual named directories within the specified path.
    github_token
        The GitHub token used to authenticate with the GitHub API. This is required
        when downloading artifacts from a different repository or from a different
        workflow run. If this is not specified, the action will attempt to download
        artifacts from the current repository and the current workflow run.
    repository
        The repository owner and the repository name joined together by ``/``. If
        ``github_token`` is specified, this is the repository that artifacts will be
        downloaded from.
    run_id
        The id of the workflow run where the desired download artifact was uploaded
        from. If ``github_token`` is specified, this is the run that artifacts will be
        downloaded from.
    args
        The inputs for a Docker container which are passed to the container's entrypoint.
        This is a subkey of the ``with`` key of the generated step.
    entrypoint
        Overrides the Docker ENTRYPOINT in the action's Dockerfile or sets one if it was not
        specified. Accepts a single string defining the executable to run (note that this is
        different from Docker's ENTRYPOINT instruction which has both a shell and exec form).
        This is a subkey of the ``with`` key of the generated step.
    condition
        A boolean expression which must be met for the step to run. Note that this represents
        the ``if`` key in the actual YAML file.
    id
        A unique identifier for the step which can be referenced in expressions.
    env
        Used to specify environment variables for the step.
    continue_on_error
        Prevents the job from failing if this step fails.
    timeout_minutes
        The maximum number of minutes to let the step run before GitHub automatically
        cancels it (defaults to 360 if not specified).

    Returns
    -------
    Step
        The generated download artifact step.

    See Also
    --------
    GitHub repository: https://github.com/actions/download-artifact
    """
    options: dict[str, object] = {
        'name': artifact_name,
        'artifact-ids': ','.join(str(s) for s in artifact_ids)
        if artifact_ids is not None
        else None,
        'pattern': pattern,
        'path': path,
        'merge-multiple': merge_multiple,
        'github-token': github_token,
        'repository': repository,
        'run-id': run_id,
    }
    options = {key: value for key, value in options.items() if value is not None}

    if name is None:
        artifact_str = check_string(options.get('artifact_name'))
        if artifact_str:
            name = f'Download {artifact_str}'
        else:
            name = 'Download Artifact'

    return action(
        name,
        'actions/download-artifact',
        ref=version,
        with_opts=options or None,
        args=args,
        entrypoint=entrypoint,
        condition=condition,
        id=id,
        env=env,
        continue_on_error=continue_on_error,
        timeout_minutes=timeout_minutes,
    )



from __future__ import annotations

from dataclasses import dataclass
from typing import TYPE_CHECKING

from ...expressions import context, StringExpression
from ..._yamloom import Step
from ..._yamloom import action
from ..types import (
    Oboollike,
    Oboolstr,
    Ointlike,
    Ostr,
    Ostrlike,
    StringLike,
)

if TYPE_CHECKING:
    from collections.abc import Mapping


__all__ = ['attest_build_provenance', 'AttestBuildProvenanceOutput']


@dataclass(frozen=True)
class AttestBuildProvenanceOutput:
    """Typed access to outputs produced by the attest_build_provenance step.

    Parameters
    ----------
    id
        The ``id`` of the attest_build_provenance step whose outputs should be
        referenced. This should match the ``id`` passed to
        :func:`attest_build_provenance`.

    Attributes
    ----------
    bundle_path
        The path to the file containing the attestation bundle.
    attestation_id
        The ID of the attestation.
    attestation_url
        The URL for the attestation summary.

    See Also
    --------
    GitHub repository: https://github.com/actions/attest-build-provenance
    """

    id: str

    @property
    def bundle_path(self) -> StringExpression:
        return context.steps[self.id].outputs['bundle-path']

    @property
    def attestation_id(self) -> StringExpression:
        return context.steps[self.id].outputs['attestation-id']

    @property
    def attestation_url(self) -> StringExpression:
        return context.steps[self.id].outputs['attestation-url']


def attest_build_provenance(
    *,
    name: Ostrlike = None,
    version: str = 'v3',
    subject_path: Ostrlike = None,
    subject_digest: Ostrlike = None,
    subject_checksums: Ostrlike = None,
    subject_name: Ostrlike = None,
    push_to_registry: Oboollike = None,
    create_storage_record: Oboollike = None,
    show_summary: Oboollike = None,
    github_token: Ostrlike = None,
    args: Ostrlike = None,
    entrypoint: Ostrlike = None,
    condition: Oboolstr = None,
    id: Ostr = None,  # noqa: A002
    env: Mapping[str, StringLike] | None = None,
    continue_on_error: Oboollike = None,
    timeout_minutes: Ointlike = None,
) -> Step:
    """Generate provenance attestations for build artifacts.

    Parameters
    ----------
    name
        The name of the step to display on GitHub.
    version
        The branch, ref, or SHA of the action's repository to use.
    subject_path
        Path to the artifact serving as the subject of the attestation. Must
        specify exactly one of ``subject_path``, ``subject_digest``, or
        ``subject_checksums``. May contain a glob pattern or list of paths (total
        subject count cannot exceed 1024).
    subject_digest
        Digest of the subject for which provenance will be generated. Must be in
        the form ``algorithm:hex_digest`` (e.g. ``sha256:abc123...``). Must specify
        exactly one of ``subject_path``, ``subject_digest``, or
        ``subject_checksums``.
    subject_name
        Subject name as it should appear in the attestation. Required when
        identifying the subject with ``subject_digest``.
    subject_checksums
        Path to checksums file containing digest and name of subjects for
        attestation. Must specify exactly one of ``subject_path``,
        ``subject_digest``, or ``subject_checksums``.
    push_to_registry
        Whether to push the provenance statement to the image registry. Requires
        that ``subject_name`` specify the fully-qualified image name and that
        ``subject_digest`` be specified. Defaults to false.
    create_storage_record
        Whether to create a storage record for the artifact. Requires that
        ``push_to_registry`` is set to true. Defaults to true.
    show_summary
        Whether to attach a list of generated attestations to the workflow run
        summary page. Defaults to true.
    github_token
        The GitHub token used to make authenticated API requests.
    args
        The inputs for a Docker container which are passed to the container's entrypoint.
        This is a subkey of the ``with`` key of the generated step.
    entrypoint
        Overrides the Docker ENTRYPOINT in the action's Dockerfile or sets one if it was not
        specified. Accepts a single string defining the executable to run (note that this is
        different from Docker's ENTRYPOINT instruction which has both a shell and exec form).
        This is a subkey of the ``with`` key of the generated step.
    condition
        A boolean expression which must be met for the step to run. Note that this represents
        the ``if`` key in the actual YAML file.
    id
        A unique identifier for the step which can be referenced in expressions.
    env
        Used to specify environment variables for the step.
    continue_on_error
        Prevents the job from failing if this step fails.
    timeout_minutes
        The maximum number of minutes to let the step run before GitHub automatically
        cancels it (defaults to 360 if not specified).

    Returns
    -------
    Step
        The generated attest build provenance step.

    See Also
    --------
    GitHub repository: https://github.com/actions/attest-build-provenance
    """
    if (
        sum(
            value is not None
            for value in (subject_path, subject_digest, subject_checksums)
        )
        != 1
    ):
        raise ValueError(
            'Exactly one of subject_path, subject_digest, or subject_checksums must be set'
        )

    options: dict[str, object] = {
        'subject-path': subject_path,
        'subject-digest': subject_digest,
        'subject-checksums': subject_checksums,
        'subject-name': subject_name,
        'push-to-registry': push_to_registry,
        'create-storage-record': create_storage_record,
        'show-summary': show_summary,
        'github-token': github_token,
    }

    options = {key: value for key, value in options.items() if value is not None}

    if name is None:
        name = 'Create Attestation'

    return action(
        name,
        'actions/attest-build-provenance',
        ref=version,
        with_opts=options or None,
        args=args,
        entrypoint=entrypoint,
        condition=condition,
        id=id,
        env=env,
        continue_on_error=continue_on_error,
        timeout_minutes=timeout_minutes,
    )



import pytest

from yamloom import Job, WorkflowInput, script
from yamloom.expressions import context


def test_job_requires_runs_on_or_uses() -> None:
    with pytest.raises(Exception):
        Job(steps=[script('echo hi')])


def test_job_rejects_runs_on_and_uses() -> None:
    with pytest.raises(Exception):
        Job(
            steps=[script('echo hi')],
            runs_on='ubuntu-latest',
            uses='org/repo/.github/workflows/reuse.yml@v1',
        )


def test_job_rejects_uses_with_steps() -> None:
    with pytest.raises(Exception):
        Job(
            steps=[script('echo hi')],
            uses='org/repo/.github/workflows/reuse.yml@v1',
        )


def test_job_rejects_runs_on_without_steps() -> None:
    with pytest.raises(Exception):
        Job(runs_on='ubuntu-latest')


def test_job_rejects_runs_on_with_empty_steps() -> None:
    with pytest.raises(Exception):
        Job(steps=[], runs_on='ubuntu-latest')


def test_job_rejects_runs_on_with_secrets_context() -> None:
    with pytest.raises(Exception):
        Job(steps=[script('echo hi')], runs_on=context.secrets.github_token)


def test_workflow_call_input_default_rejects_secrets() -> None:
    with pytest.raises(Exception):
        WorkflowInput.string(default=context.secrets.github_token)


def test_workflow_call_input_default_allows_github() -> None:
    WorkflowInput.string(default=context.github.actor)



from __future__ import annotations
from yamloom.actions.utils import validate_choice

from dataclasses import dataclass
from typing import TYPE_CHECKING

from ...expressions import context, StringExpression
from ..._yamloom import Step
from ..._yamloom import action
from ..types import (
    Oboollike,
    Oboolstr,
    Ointlike,
    Ostr,
    Ostrlike,
    StringLike,
    StringOrBoolLike,
)

if TYPE_CHECKING:
    from collections.abc import Mapping

__all__ = ['setup_python', 'setup_uv', 'SetupPythonOutput', 'SetupUvOutput']


@dataclass(frozen=True)
class SetupPythonOutput:
    """Typed access to outputs produced by the setup_python step.

    Parameters
    ----------
    id
        The ``id`` of the setup_python step whose outputs should be referenced.
        This should match the ``id`` passed to :func:`setup_python`.

    Attributes
    ----------
    python_version
        The installed Python or PyPy version.
    cache_hit
        A boolean value to indicate a cache entry was found.
    python_path
        The absolute path to the Python or PyPy executable.

    See Also
    --------
    GitHub repository: https://github.com/actions/setup-python
    """

    id: str

    @property
    def python_version(self) -> StringExpression:
        return context.steps[self.id].outputs['python-version']

    @property
    def cache_hit(self) -> StringExpression:
        return context.steps[self.id].outputs['cache-hit']

    @property
    def python_path(self) -> StringExpression:
        return context.steps[self.id].outputs['python-path']


@dataclass(frozen=True)
class SetupUvOutput:
    """Typed access to outputs produced by the setup_uv step.

    Parameters
    ----------
    id
        The ``id`` of the setup_uv step whose outputs should be referenced.
        This should match the ``id`` passed to :func:`setup_uv`.

    Attributes
    ----------
    uv_version
        The installed uv version.
    uv_path
        The path to the installed uv binary.
    uvx_path
        The path to the installed uvx binary.
    cache_hit
        A boolean value to indicate a cache entry was found.
    cache_key
        The cache key used for storing/restoring the cache.
    venv
        Path to the activated venv if activate-environment is true.
    python_version
        The Python version that was set.
    python_cache_hit
        A boolean value to indicate the Python cache entry was found.

    See Also
    --------
    GitHub repository: https://github.com/astral-sh/setup-uv
    """

    id: str

    @property
    def uv_version(self) -> StringExpression:
        return context.steps[self.id].outputs['uv-version']

    @property
    def uv_path(self) -> StringExpression:
        return context.steps[self.id].outputs['uv-path']

    @property
    def uvx_path(self) -> StringExpression:
        return context.steps[self.id].outputs['uvx-path']

    @property
    def cache_hit(self) -> StringExpression:
        return context.steps[self.id].outputs['cache-hit']

    @property
    def cache_key(self) -> StringExpression:
        return context.steps[self.id].outputs['cache-key']

    @property
    def venv(self) -> StringExpression:
        return context.steps[self.id].outputs.venv

    @property
    def python_version(self) -> StringExpression:
        return context.steps[self.id].outputs['python-version']

    @property
    def python_cache_hit(self) -> StringExpression:
        return context.steps[self.id].outputs['python-cache-hit']


def setup_python(
    *,
    name: Ostrlike = None,
    version: str = 'v6',
    python_version: StringLike | None = None,
    python_version_file: Ostrlike = None,
    check_latest: Oboollike = None,
    architecture: Ostrlike = None,
    token: Ostrlike = None,
    cache: Ostrlike = None,
    cache_dependency_path: Ostrlike = None,
    update_environment: Oboollike = None,
    allow_prereleases: Oboollike = None,
    freethreaded: Oboollike = None,
    pip_version: Ostrlike = None,
    pip_install: Ostrlike = None,
    args: Ostrlike = None,
    entrypoint: Ostrlike = None,
    condition: Oboolstr = None,
    id: Ostr = None,  # noqa: A002
    env: Mapping[str, StringLike] | None = None,
    continue_on_error: Oboollike = None,
    timeout_minutes: Ointlike = None,
) -> Step:
    """Set up a specific version of Python and add it to the PATH.

    Parameters
    ----------
    name
        The name of the step to display on GitHub.
    version
        The branch, ref, or SHA of the action's repository to use.
    python_version
        Version range or exact version of Python or PyPy to use.
    python_version_file
        File containing the Python version to use.
    cache
        Package manager for caching in the default directory. Supported values:
        ``pip``, ``pipenv``, ``poetry``.
    architecture
        The target architecture of the Python or PyPy interpreter.
    check_latest
        Check for the latest available version that satisfies the version spec.
    token
        Token used to authenticate when fetching Python distributions.
    cache_dependency_path
        Path to dependency files. Supports wildcards or a list of file names.
    update_environment
        Set this option to update environment variables.
    allow_prereleases
        Allow prerelease versions to satisfy version range.
    freethreaded
        Use the freethreaded version of Python.
    pip_versions
        The version of pip to install with Python.
    pip_install
        Packages to install with pip after setting up Python.
    args
        The inputs for a Docker container which are passed to the container's entrypoint.
        This is a subkey of the ``with`` key of the generated step.
    entrypoint
        Overrides the Docker ENTRYPOINT in the action's Dockerfile or sets one if it was not
        specified. Accepts a single string defining the executable to run (note that this is
        different from Docker's ENTRYPOINT instruction which has both a shell and exec form).
        This is a subkey of the ``with`` key of the generated step.
    condition
        A boolean expression which must be met for the step to run. Note that this represents
        the ``if`` key in the actual YAML file.
    id
        A unique identifier for the step which can be referenced in expressions.
    env
        Used to specify environment variables for the step.
    continue_on_error
        Prevents the job from failing if this step fails.
    timeout_minutes
        The maximum number of minutes to let the step run before GitHub automatically
        cancels it (defaults to 360 if not specified).

    Returns
    -------
    Step
        The generated setup-python step.

    See Also
    --------
    GitHub repository: https://github.com/actions/setup-python
    """
    options: dict[str, object] = {
        'python-version': python_version,
        'python-version-file': python_version_file,
        'check-latest': check_latest,
        'architecture': architecture,
        'token': token,
        'cache': cache,
        'cache-dependency-path': cache_dependency_path,
        'update-environment': update_environment,
        'allow-prereleases': allow_prereleases,
        'freethreaded': freethreaded,
        'pip-version': pip_version,
        'pip-install': pip_install,
    }

    if cache is not None:
        if isinstance(cache, str):
            lowered = cache.lower()
            if lowered not in {'pip', 'pipenv', 'poetry'}:
                msg = "'cache' must be 'pip', 'pipenv' or 'poetry'"
                raise ValueError(msg)
            options['cache'] = lowered
        else:
            options['cache'] = cache

    options = {key: value for key, value in options.items() if value is not None}

    if name is None:
        name = 'Setup Python'

    return action(
        name,
        'actions/setup-python',
        ref=version,
        with_opts=options or None,
        args=args,
        entrypoint=entrypoint,
        condition=condition,
        id=id,
        env=env,
        continue_on_error=continue_on_error,
        timeout_minutes=timeout_minutes,
    )


def setup_uv(
    *,
    name: Ostrlike = None,
    version: str = 'v7',
    uv_version: Ostrlike = None,
    uv_version_file: Ostrlike = None,
    resolution_strategy: Ostrlike = None,
    python_version: Ostrlike = None,
    activate_environment: Oboollike = None,
    working_directory: Ostrlike = None,
    checksum: Ostrlike = None,
    github_token: Ostrlike = None,
    enable_cache: StringOrBoolLike | None = None,
    cache_dependency_glob: list[StringLike] | None = None,
    restore_cache: Oboollike = None,
    save_cache: Oboollike = None,
    cache_suffix: Ostrlike = None,
    cache_local_path: Ostrlike = None,
    prune_cache: Oboollike = None,
    cache_python: Oboollike = None,
    ignore_nothing_to_cache: Oboollike = None,
    ignore_empty_workdir: Oboollike = None,
    tool_dir: Ostrlike = None,
    tool_bin_dir: Ostrlike = None,
    manifest_file: Ostrlike = None,
    add_problem_matchers: Oboollike = None,
    args: Ostrlike = None,
    entrypoint: Ostrlike = None,
    condition: Oboolstr = None,
    id: Ostr = None,  # noqa: A002
    env: Mapping[str, StringLike] | None = None,
    continue_on_error: Oboollike = None,
    timeout_minutes: Ointlike = None,
) -> Step:
    """Set up a specific version of uv.

    Parameters
    ----------
    name
        The name of the step to display on GitHub.
    version
        The branch, ref, or SHA of the action's repository to use.
    uv_version
        The version of uv to install.
    uv_version_file
        Path to a file containing the version of uv to install.
    python_version
        The version of Python to set UV_PYTHON to.
    activate_environment
        Use uv venv to activate a venv ready to be used by later steps.
    working_directory
        The directory to execute all commands in and look for files.
    checksum
        The checksum of the uv version to install.
    github_token
        Used to increase the rate limit when retrieving versions and downloading uv.
    enable_cache
        Enable uploading of the uv cache. Accepts ``true``, ``false``, or ``auto``.
    cache_dependency_glob
        Glob pattern list to control the cache.
    restore_cache
        Whether to restore the cache if found.
    save_cache
        Whether to save the cache after the run.
    cache_suffix
        Suffix for the cache key.
    cache_local_path
        Local path to store the cache.
    prune_cache
        Prune cache before saving.
    cache_python
        Upload managed Python installations to the GitHub Actions cache.
    ignore_nothing_to_cache
        Ignore when nothing is found to cache.
    ignore_empty_workdir
        Ignore when the working directory is empty.
    tool_dir
        Custom path to set UV_TOOL_DIR to.
    tool_bin_dir
        Custom path to set UV_TOOL_BIN_DIR to.
    manifest_file
        URL to the manifest file containing available versions and download URLs.
    add_problem_matchers
        Add problem matchers.
    resolution_strategy
        Resolution strategy when resolving version ranges (``highest`` or
        ``lowest``).
    args
        The inputs for a Docker container which are passed to the container's entrypoint.
        This is a subkey of the ``with`` key of the generated step.
    entrypoint
        Overrides the Docker ENTRYPOINT in the action's Dockerfile or sets one if it was not
        specified. Accepts a single string defining the executable to run (note that this is
        different from Docker's ENTRYPOINT instruction which has both a shell and exec form).
        This is a subkey of the ``with`` key of the generated step.
    condition
        A boolean expression which must be met for the step to run. Note that this represents
        the ``if`` key in the actual YAML file.
    id
        A unique identifier for the step which can be referenced in expressions.
    env
        Used to specify environment variables for the step.
    continue_on_error
        Prevents the job from failing if this step fails.
    timeout_minutes
        The maximum number of minutes to let the step run before GitHub automatically
        cancels it (defaults to 360 if not specified).

    Returns
    -------
    Step
        The generated setup-uv step.

    See Also
    --------
    GitHub repository: https://github.com/astral-sh/setup-uv
    """
    options: dict[str, object] = {
        'version': uv_version,
        'version-file': uv_version_file,
        'resolution-strategy': validate_choice(
            'resolution_strategy', resolution_strategy, ['highest', 'lowest']
        ),
        'python-version': python_version,
        'activate-environment': activate_environment,
        'working-directory': working_directory,
        'checksum': checksum,
        'github-token': github_token,
        'enable-cache': enable_cache,
        'cache-dependency-glob': '\n'.join(str(s) for s in cache_dependency_glob)
        if cache_dependency_glob is not None
        else None,
        'restore-cache': restore_cache,
        'save-cache': save_cache,
        'cache-suffix': cache_suffix,
        'cache-local-path': cache_local_path,
        'prune-cache': prune_cache,
        'cache-python': cache_python,
        'ignore-nothing-to-cache': ignore_nothing_to_cache,
        'ignore-empty-workdir': ignore_empty_workdir,
        'tool-dir': tool_dir,
        'tool-bin-dir': tool_bin_dir,
        'manifest-file': manifest_file,
        'add-problem-matchers': add_problem_matchers,
    }

    if enable_cache is not None:
        if isinstance(enable_cache, str):
            if enable_cache.lower() != 'auto':
                msg = "'enable_cache' must be 'auto', true or false"
                raise ValueError(msg)
            options['enable-cache'] = 'auto'
        elif isinstance(enable_cache, bool):
            options['enable-cache'] = enable_cache
        else:
            options['enable-cache'] = enable_cache

    options = {key: value for key, value in options.items() if value is not None}

    if name is None:
        name = 'Setup uv'

    return action(
        name,
        'astral-sh/setup-uv',
        ref=version,
        with_opts=options or None,
        args=args,
        entrypoint=entrypoint,
        condition=condition,
        id=id,
        env=env,
        continue_on_error=continue_on_error,
        timeout_minutes=timeout_minutes,
    )



from __future__ import annotations
from yamloom.actions.utils import validate_choice

from dataclasses import dataclass
from typing import TYPE_CHECKING

from ...expressions import context, StringExpression
from ..._yamloom import Step
from ..._yamloom import action
from ..types import (
    Oboollike,
    Oboolstr,
    Ointlike,
    Ostr,
    Ostrlike,
    StringLike,
)

if TYPE_CHECKING:
    from collections.abc import Mapping

__all__ = ['setup_dotnet', 'SetupDotnetOutput']


@dataclass(frozen=True)
class SetupDotnetOutput:
    """Typed access to outputs produced by the setup_dotnet step.

    Parameters
    ----------
    id
        The ``id`` of the setup_dotnet step whose outputs should be referenced.
        This should match the ``id`` passed to :func:`setup_dotnet`.

    Attributes
    ----------
    cache_hit
        A boolean value to indicate if a cache was hit.
    dotnet_version
        The installed .NET SDK version.

    See Also
    --------
    GitHub repository: https://github.com/actions/setup-dotnet
    """

    id: str

    @property
    def cache_hit(self) -> StringExpression:
        return context.steps[self.id].outputs['cache-hit']

    @property
    def dotnet_version(self) -> StringExpression:
        return context.steps[self.id].outputs['dotnet-version']


def setup_dotnet(
    *,
    name: Ostrlike = None,
    version: str = 'v5',
    dotnet_version: Ostrlike = None,
    dotnet_quality: Ostrlike = None,
    global_json_file: Ostrlike = None,
    source_url: Ostrlike = None,
    owner: Ostrlike = None,
    config_file: Ostrlike = None,
    cache: Oboollike = None,
    cache_dependency_path: Ostrlike | list[StringLike] = None,
    args: Ostrlike = None,
    entrypoint: Ostrlike = None,
    condition: Oboolstr = None,
    id: Ostr = None,  # noqa: A002
    env: Mapping[str, StringLike] | None = None,
    continue_on_error: Oboollike = None,
    timeout_minutes: Ointlike = None,
) -> Step:
    """Set up a specific version of the .NET SDK and optional NuGet auth.

    Parameters
    ----------
    name
        The name of the step to display on GitHub.
    version
        The branch, ref, or SHA of the action's repository to use.
    dotnet_version
        Optional SDK version(s) to use. Examples: ``2.2.104``, ``3.1``, ``3.1.x``,
        ``3.x``, ``6.0.2xx``.
    dotnet_quality
        Optional quality of the build. Possible values are ``daily``, ``signed``,
        ``validated``, ``preview``, and ``ga``.
    global_json_file
        Optional global.json location, if your global.json isn't located in the
        root of the repo.
    source_url
        Optional package source for which to set up authentication.
    owner
        Optional owner for using packages from GitHub Package Registry
        organizations/users other than the current repository's owner. Only used
        if a GPR URL is also provided in ``source_url``.
    config_file
        Optional NuGet.config location, if your NuGet.config isn't located in the
        root of the repo.
    cache
        Enable caching of the NuGet global-packages folder.
    cache_dependency_path
        Path to a dependency file such as ``packages.lock.json``. Supports
        wildcards or a list of file names for caching multiple dependencies.
    args
        The inputs for a Docker container which are passed to the container's entrypoint.
        This is a subkey of the ``with`` key of the generated step.
    entrypoint
        Overrides the Docker ENTRYPOINT in the action's Dockerfile or sets one if it was not
        specified. Accepts a single string defining the executable to run (note that this is
        different from Docker's ENTRYPOINT instruction which has both a shell and exec form).
        This is a subkey of the ``with`` key of the generated step.
    condition
        A boolean expression which must be met for the step to run. Note that this represents
        the ``if`` key in the actual YAML file.
    id
        A unique identifier for the step which can be referenced in expressions.
    env
        Used to specify environment variables for the step.
    continue_on_error
        Prevents the job from failing if this step fails.
    timeout_minutes
        The maximum number of minutes to let the step run before GitHub automatically
        cancels it (defaults to 360 if not specified).

    Returns
    -------
    Step
        The generated setup-dotnet step.

    See Also
    --------
    GitHub repository: https://github.com/actions/setup-dotnet
    """
    options: dict[str, object] = {
        'dotnet-version': dotnet_version,
        'dotnet-quality': validate_choice(
            'dotnet-quality',
            dotnet_quality,
            ['daily', 'signed', 'validated', 'preview', 'ga'],
        ),
        'global-json-file': global_json_file,
        'source-url': source_url,
        'owner': owner,
        'config-file': config_file,
        'cache': cache,
        'cache-dependency-path': cache_dependency_path,
    }
    options = {key: value for key, value in options.items() if value is not None}

    if name is None:
        name = 'Setup .NET'

    return action(
        name,
        'actions/setup-dotnet',
        ref=version,
        with_opts=options or None,
        args=args,
        entrypoint=entrypoint,
        condition=condition,
        id=id,
        env=env,
        continue_on_error=continue_on_error,
        timeout_minutes=timeout_minutes,
    )



from __future__ import annotations

from dataclasses import dataclass
from typing import TYPE_CHECKING

from ...expressions import context, StringExpression
from ..._yamloom import Step
from ..._yamloom import action
from ..types import (
    Obool,
    Oboollike,
    Oboolstr,
    Oint,
    Ointlike,
    Ostr,
    Ostrlike,
    StringLike,
)

if TYPE_CHECKING:
    from collections.abc import Mapping

__all__ = [
    'cache',
    'cache_restore',
    'cache_save',
    'CacheOutput',
    'CacheRestoreOutput',
]


@dataclass(frozen=True)
class CacheOutput:
    """Typed access to outputs produced by the cache step.

    Parameters
    ----------
    id
        The ``id`` of the cache step whose outputs should be referenced. This
        should match the ``id`` passed to :func:`cache`.

    Attributes
    ----------
    cache_hit
        A boolean value to indicate an exact match was found for the primary key.

    See Also
    --------
    GitHub repository: https://github.com/actions/cache
    """

    id: str

    @property
    def cache_hit(self) -> StringExpression:
        return context.steps[self.id].outputs['cache-hit']


@dataclass(frozen=True)
class CacheRestoreOutput:
    """Typed access to outputs produced by the cache_restore step.

    Parameters
    ----------
    id
        The ``id`` of the cache_restore step whose outputs should be referenced.
        This should match the ``id`` passed to :func:`cache_restore`.

    Attributes
    ----------
    cache_hit
        A boolean value to indicate an exact match was found for the primary key.
    cache_primary_key
        A resolved cache key for which cache match was attempted.
    cache_matched_key
        Key of the cache that was restored, either the primary key on cache-hit
        or a partial/complete match of one of the restore keys.

    See Also
    --------
    GitHub repository: https://github.com/actions/cache
    """

    id: str

    @property
    def cache_hit(self) -> StringExpression:
        return context.steps[self.id].outputs['cache-hit']

    @property
    def cache_primary_key(self) -> StringExpression:
        return context.steps[self.id].outputs['cache-primary-key']

    @property
    def cache_matched_key(self) -> StringExpression:
        return context.steps[self.id].outputs['cache-matched-key']


def cache(
    *,
    key: str,
    name: Ostrlike = None,
    version: str = 'v5',
    path: list[str] | None = None,
    restore_keys: list[str] | None = None,
    upload_chunk_size: Ointlike = None,
    enable_cross_os_archive: Obool = None,
    fail_on_cache_miss: Obool = None,
    lookup_only: Obool = None,
    save_always: Obool = None,
    segment_download_timeout_mins: Oint = None,
    args: Ostrlike = None,
    entrypoint: Ostrlike = None,
    condition: Oboolstr = None,
    id: Ostr = None,  # noqa: A002
    env: Mapping[str, StringLike] | None = None,
    continue_on_error: Oboollike = None,
    timeout_minutes: Ointlike = None,
) -> Step:
    """Cache artifacts like dependencies and build outputs.

    Parameters
    ----------
    key
        An explicit key for restoring and saving the cache.
    name
        The name of the step to display on GitHub.
    version
        The branch, ref, or SHA of the action's repository to use.
    path
        A list of files, directories, and wildcard patterns to cache and restore.
    restore_keys
        An ordered multiline string listing the prefix-matched keys that are used
        for restoring stale cache if no cache hit occurred for ``key``. Note
        ``cache_hit`` returns false in this case.
    upload_chunk_size
        The chunk size used to split up large files during upload, in bytes.
    enable_cross_os_archive
        An optional boolean when enabled, allows Windows runners to save or
        restore caches that can be restored or saved respectively on other
        platforms.
    fail_on_cache_miss
        Fail the workflow if cache entry is not found.
    lookup_only
        Check if a cache entry exists for the given input(s) (``key``,
        ``restore_keys``) without downloading the cache.
    save_always
        Run the post step to save the cache even if another step before fails.
        This does not work as intended and will be removed in a future release.
        A separate ``actions/cache/restore`` step should be used instead.
    segment_download_timeout_mins
        The download segment timeout (in minutes) used by the cache action.
    args
        The inputs for a Docker container which are passed to the container's entrypoint.
        This is a subkey of the ``with`` key of the generated step.
    entrypoint
        Overrides the Docker ENTRYPOINT in the action's Dockerfile or sets one if it was not
        specified. Accepts a single string defining the executable to run (note that this is
        different from Docker's ENTRYPOINT instruction which has both a shell and exec form).
        This is a subkey of the ``with`` key of the generated step.
    condition
        A boolean expression which must be met for the step to run. Note that this represents
        the ``if`` key in the actual YAML file.
    id
        A unique identifier for the step which can be referenced in expressions.
    env
        Used to specify environment variables for the step.
    continue_on_error
        Prevents the job from failing if this step fails.
    timeout_minutes
        The maximum number of minutes to let the step run before GitHub automatically
        cancels it (defaults to 360 if not specified).

    Returns
    -------
    Step
        The generated cache step.

    See Also
    --------
    GitHub repository: https://github.com/actions/cache
    """
    options: dict[str, object] = {
        'key': key,
        'path': list(path) if path is not None else None,
        'restore-keys': list(restore_keys) if restore_keys is not None else None,
        'upload-chunk-size': upload_chunk_size,
        'enableCrossOsArchive': enable_cross_os_archive,
        'fail-on-cache-miss': fail_on_cache_miss,
        'lookup-only': lookup_only,
        'save-always': save_always,
    }
    options = {key: value for key, value in options.items() if value is not None}

    if name is None:
        name = 'Save/Restore Cache'

    if segment_download_timeout_mins is not None:
        merged_env = dict(env or {})
        merged_env['SEGMENT_DOWNLOAD_TIMEOUT_MINS'] = str(segment_download_timeout_mins)
        env = merged_env

    return action(
        name,
        'actions/cache',
        ref=version,
        with_opts=options or None,
        args=args,
        entrypoint=entrypoint,
        condition=condition,
        id=id,
        env=env,
        continue_on_error=continue_on_error,
        timeout_minutes=timeout_minutes,
    )


def cache_save(
    *,
    key: str,
    name: Ostrlike = None,
    version: str = 'v5',
    path: list[str] | None = None,
    upload_chunk_size: Ointlike = None,
    enable_cross_os_archive: Obool = None,
    segment_download_timeout_mins: Oint = None,
    args: Ostrlike = None,
    entrypoint: Ostrlike = None,
    condition: Oboolstr = None,
    id: Ostr = None,  # noqa: A002
    env: Mapping[str, StringLike] | None = None,
    continue_on_error: Oboollike = None,
    timeout_minutes: Ointlike = None,
) -> Step:
    """Save cache artifacts like dependencies and build outputs.

    Parameters
    ----------
    key
        An explicit key for saving the cache.
    name
        The name of the step to display on GitHub.
    version
        The branch, ref, or SHA of the action's repository to use.
    path
        A list of files, directories, and wildcard patterns to cache.
    upload_chunk_size
        The chunk size used to split up large files during upload, in bytes.
    enable_cross_os_archive
        An optional boolean when enabled, allows Windows runners to save caches
        that can be restored on other platforms.
    segment_download_timeout_mins
        The download segment timeout (in minutes) used by the cache action.
    args
        The inputs for a Docker container which are passed to the container's entrypoint.
        This is a subkey of the ``with`` key of the generated step.
    entrypoint
        Overrides the Docker ENTRYPOINT in the action's Dockerfile or sets one if it was not
        specified. Accepts a single string defining the executable to run (note that this is
        different from Docker's ENTRYPOINT instruction which has both a shell and exec form).
        This is a subkey of the ``with`` key of the generated step.
    condition
        A boolean expression which must be met for the step to run. Note that this represents
        the ``if`` key in the actual YAML file.
    id
        A unique identifier for the step which can be referenced in expressions.
    env
        Used to specify environment variables for the step.
    continue_on_error
        Prevents the job from failing if this step fails.
    timeout_minutes
        The maximum number of minutes to let the step run before GitHub automatically
        cancels it (defaults to 360 if not specified).

    Returns
    -------
    Step
        The generated cache save step.

    See Also
    --------
    GitHub repository: https://github.com/actions/cache
    """
    options: dict[str, object] = {
        'key': key,
        'path': list(path) if path is not None else None,
        'upload-chunk-size': upload_chunk_size,
        'enableCrossOsArchive': enable_cross_os_archive,
    }
    options = {key: value for key, value in options.items() if value is not None}

    if name is None:
        name = 'Save Cache'

    if segment_download_timeout_mins is not None:
        merged_env = dict(env or {})
        merged_env['SEGMENT_DOWNLOAD_TIMEOUT_MINS'] = str(segment_download_timeout_mins)
        env = merged_env

    return action(
        name,
        'actions/cache/save',
        ref=version,
        with_opts=options or None,
        args=args,
        entrypoint=entrypoint,
        condition=condition,
        id=id,
        env=env,
        continue_on_error=continue_on_error,
        timeout_minutes=timeout_minutes,
    )


def cache_restore(
    *,
    key: StringLike,
    name: Ostrlike = None,
    version: str = 'v5',
    path: list[StringLike] | None = None,
    restore_keys: list[StringLike] | None = None,
    enable_cross_os_archive: Oboollike = None,
    fail_on_cache_miss: Oboollike = None,
    lookup_only: Oboollike = None,
    segment_download_timeout_mins: Ointlike = None,
    args: Ostrlike = None,
    entrypoint: Ostrlike = None,
    condition: Oboolstr = None,
    id: Ostr = None,  # noqa: A002
    env: Mapping[str, StringLike] | None = None,
    continue_on_error: Oboollike = None,
    timeout_minutes: Ointlike = None,
) -> Step:
    """Restore cache artifacts like dependencies and build outputs.

    Parameters
    ----------
    key
        An explicit key for restoring the cache.
    name
        The name of the step to display on GitHub.
    version
        The branch, ref, or SHA of the action's repository to use.
    path
        A list of files, directories, and wildcard patterns to restore.
    restore_keys
        An ordered multiline string listing the prefix-matched keys that are used
        for restoring stale cache if no cache hit occurred for ``key``. Note
        ``cache_hit`` returns false in this case.
    enable_cross_os_archive
        An optional boolean when enabled, allows Windows runners to restore caches
        that were saved on other platforms.
    fail_on_cache_miss
        Fail the workflow if cache entry is not found.
    lookup_only
        Check if a cache entry exists for the given input(s) (``key``,
        ``restore_keys``) without downloading the cache.
    segment_download_timeout_mins
        The download segment timeout (in minutes) used by the cache action.
    args
        The inputs for a Docker container which are passed to the container's entrypoint.
        This is a subkey of the ``with`` key of the generated step.
    entrypoint
        Overrides the Docker ENTRYPOINT in the action's Dockerfile or sets one if it was not
        specified. Accepts a single string defining the executable to run (note that this is
        different from Docker's ENTRYPOINT instruction which has both a shell and exec form).
        This is a subkey of the ``with`` key of the generated step.
    condition
        A boolean expression which must be met for the step to run. Note that this represents
        the ``if`` key in the actual YAML file.
    id
        A unique identifier for the step which can be referenced in expressions.
    env
        Used to specify environment variables for the step.
    continue_on_error
        Prevents the job from failing if this step fails.
    timeout_minutes
        The maximum number of minutes to let the step run before GitHub automatically
        cancels it (defaults to 360 if not specified).

    Returns
    -------
    Step
        The generated cache restore step.

    See Also
    --------
    GitHub repository: https://github.com/actions/cache
    """
    options: dict[str, object] = {
        'key': key,
        'path': list(path) if path is not None else None,
        'restore-keys': list(restore_keys) if restore_keys is not None else None,
        'enableCrossOsArchive': enable_cross_os_archive,
        'fail-on-cache-miss': fail_on_cache_miss,
        'lookup-only': lookup_only,
    }
    options = {key: value for key, value in options.items() if value is not None}

    if name is None:
        name = 'Restore Cache'

    if segment_download_timeout_mins is not None:
        merged_env = dict(env or {})
        merged_env['SEGMENT_DOWNLOAD_TIMEOUT_MINS'] = str(segment_download_timeout_mins)
        env = merged_env

    return action(
        name,
        'actions/cache/restore',
        ref=version,
        with_opts=options or None,
        args=args,
        entrypoint=entrypoint,
        condition=condition,
        id=id,
        env=env,
        continue_on_error=continue_on_error,
        timeout_minutes=timeout_minutes,
    )



from __future__ import annotations
from yamloom.actions.utils import validate_choice

from dataclasses import dataclass
from typing import TYPE_CHECKING

from ...expressions import context, StringExpression
from ..._yamloom import Step
from ..._yamloom import action
from ..types import (
    Oboollike,
    Oboolstr,
    Ointlike,
    Ostr,
    Ostrlike,
    StringLike,
)

if TYPE_CHECKING:
    from collections.abc import Mapping

__all__ = ['setup_php', 'SetupPhpOutput']


@dataclass(frozen=True)
class SetupPhpOutput:
    """Typed access to outputs produced by the setup_php step.

    Parameters
    ----------
    id
        The ``id`` of the setup_php step whose outputs should be referenced.
        This should match the ``id`` passed to :func:`setup_php`.

    Attributes
    ----------
    php_version
        PHP version in semver format.

    See Also
    --------
    GitHub repository: https://github.com/shivammathur/setup-php
    """

    id: str

    @property
    def php_version(self) -> StringExpression:
        return context.steps[self.id].outputs['php-version']


def setup_php(
    *,
    name: Ostrlike = None,
    version: str = 'v2',
    php_version: Ostrlike = None,
    php_version_file: Ostrlike = None,
    extensions: Ostrlike = None,
    ini_file: Ostrlike = None,
    ini_values: Ostrlike = None,
    coverage: Ostrlike = None,
    tools: Ostrlike = None,
    github_token: Ostrlike = None,
    args: Ostrlike = None,
    entrypoint: Ostrlike = None,
    condition: Oboolstr = None,
    id: Ostr = None,  # noqa: A002
    env: Mapping[str, StringLike] | None = None,
    continue_on_error: Oboollike = None,
    timeout_minutes: Ointlike = None,
) -> Step:
    """Set up PHP.

    Parameters
    ----------
    name
        The name of the step to display on GitHub.
    version
        The branch, ref, or SHA of the action's repository to use.
    php_version
        Setup PHP version.
    php_version_file
        Setup PHP version from a file.
    extensions
        Setup PHP extensions.
    ini_file
        Set base ini file.
    ini_values
        Add values to php.ini.
    coverage
        Setup code coverage driver.
    tools
        Setup popular tools globally.
    github_token
        GitHub token to use for authentication.
    args
        The inputs for a Docker container which are passed to the container's entrypoint.
        This is a subkey of the ``with`` key of the generated step.
    entrypoint
        Overrides the Docker ENTRYPOINT in the action's Dockerfile or sets one if it was not
        specified. Accepts a single string defining the executable to run (note that this is
        different from Docker's ENTRYPOINT instruction which has both a shell and exec form).
        This is a subkey of the ``with`` key of the generated step.
    condition
        A boolean expression which must be met for the step to run. Note that this represents
        the ``if`` key in the actual YAML file.
    id
        A unique identifier for the step which can be referenced in expressions.
    env
        Used to specify environment variables for the step.
    continue_on_error
        Prevents the job from failing if this step fails.
    timeout_minutes
        The maximum number of minutes to let the step run before GitHub automatically
        cancels it (defaults to 360 if not specified).

    Returns
    -------
    Step
        The generated setup-php step.

    See Also
    --------
    GitHub repository: https://github.com/shivammathur/setup-php
    """
    options: dict[str, object] = {
        'php-version': php_version,
        'php-version-file': php_version_file,
        'extensions': extensions,
        'ini-file': validate_choice(
            'ini_file', ini_file, ['production', 'development', 'none']
        ),
        'ini-values': ini_values,
        'coverage': validate_choice('coverage', coverage, ['xdebug', 'pcov', 'none']),
        'tools': tools,
        'github-token': github_token,
    }
    options = {key: value for key, value in options.items() if value is not None}

    if name is None:
        name = 'Setup PHP'

    return action(
        name,
        'shivammathur/setup-php',
        ref=version,
        with_opts=options or None,
        args=args,
        entrypoint=entrypoint,
        condition=condition,
        id=id,
        env=env,
        continue_on_error=continue_on_error,
        timeout_minutes=timeout_minutes,
    )
